@@ -7,6 +7,8 @@ from fsspec.implementations.local import LocalFileSystem
 from fsspec.spec import AbstractFileSystem
 from pyarrow.dataset import dataset as pyarrow_dataset
 
+from deltalake.schema import schema_from_string
+
 
 class DeltaTable:
     """
@@ -31,8 +33,6 @@ class DeltaTable:
         self.version = 0
         self.checkpoint = 0
         self.files = set()
-        self._latest_file = ""
-        self._latest_file_time = 0
         if file_system is None:
             file_system = LocalFileSystem()
         self.filesystem = file_system
@@ -44,34 +44,19 @@ class DeltaTable:
         self.pyarrow_dataset = self._pyarrow_dataset()
 
     def _pyarrow_dataset(self):
-        # Use the latest file to fetch the schema.
-        # This will allow for schema evolution
-        schema = pyarrow_dataset(
-            source=self._latest_file,
-            filesystem=self.filesystem,
-            partitioning="hive",
-            format="parquet",
-        ).schema
-
         return pyarrow_dataset(
             source=list(self.files),
             filesystem=self.filesystem,
             partitioning="hive",
             format="parquet",
-            schema=schema,
+            schema=self.schema,
         )
 
-    @property
-    def schema(self):
-        return self.pyarrow_dataset.schema
-
     def _is_delta_table(self):
         return self.filesystem.exists(f"{self.log_path}/{0:020}.json")
 
     def _reset_state(self):
         self.files = set()
-        self._latest_file = ""
-        self._latest_file_time = 0
 
     def _apply_from_checkpoint(self, checkpoint_version: int):
 
@@ -82,23 +67,22 @@ class DeltaTable:
         if self.checkpoint == 0:
             return
 
-        # read latest checkpoint
-        with self.filesystem.open(
-            f"{self.log_path}/{self.checkpoint:020}.checkpoint.parquet"
-        ) as checkpoint_file:
-            checkpoint = pq.read_table(checkpoint_file).to_pandas()
+        # read latest checkpoint. It might be a multi-part checkpoint for very large checkpoints
+        checkpoint_files = self.filesystem.glob(
+            f"{self.log_path}/{self.checkpoint:020}.checkpoint*.parquet"
+        )
+        for checkpoint_path in checkpoint_files:
+            with self.filesystem.open(checkpoint_path) as checkpoint_file:
+                checkpoint = pq.read_table(checkpoint_file).to_pandas()
 
-            for i, row in checkpoint.iterrows():
-                if not row["add"]:
-                    continue
-                added_file = f"{self.path}/{row['add']['path']}"
-                if added_file:
-                    self.files.add(added_file)
-
-                modificationTime = row["add"]["modificationTime"]
-                if modificationTime > self._latest_file_time:
-                    self._latest_file = added_file
-                    self._latest_file_time = modificationTime
+                for _, row in checkpoint.iterrows():
+                    if row["metaData"]:
+                        schema_string = row["metaData"]["schemaString"]
+                        self.schema = schema_from_string(schema_string)
+                    elif row["add"]:
+                        added_file = f"{self.path}/{row['add']['path']}"
+                        if added_file:
+                            self.files.add(added_file)
 
     def _apply_partial_logs(self, version: int):
         # Checkpoints are created every 10 transactions,
@@ -122,16 +106,14 @@ class DeltaTable:
             with self.filesystem.open(log_file) as log:
                 for line in log:
                     meta_data = json.loads(line)
+
                     # Log contains other stuff, but we are only
                     # interested in the add or remove entries
                     if "add" in meta_data.keys():
                         file = f"{self.path}/{meta_data['add']['path']}"
                         self.files.add(file)
 
-                        self._latest_file_time = meta_data["add"]["modificationTime"]
-                        self._latest_file = file
-
-                    if "remove" in meta_data.keys():
+                    elif "remove" in meta_data.keys():
                         remove_file = f"{self.path}/{meta_data['remove']['path']}"
                         # To handle 0 checkpoints, we might read the log file with
                         # same version as checkpoint. this means that we try to
@@ -139,6 +121,9 @@ class DeltaTable:
                         # which we don't have in the list
                         if remove_file in self.files:
                             self.files.remove(remove_file)
+                    elif "metaData" in meta_data.keys():
+                        schema_string = meta_data["metaData"]["schemaString"]
+                        self.schema = schema_from_string(schema_string)
                 # Stop if we have reatched the desired version
                 if self.version == version:
                     break
